/* ===========================================================================

   The Millennium Operating System
   This file is part of the Millennium Operating System source code.
   ConioMMK.c: Standard Function Calls to manipulate the console
   Necessary code to manipulate the console (cursor,colors,switching,etc)

   First functional test code by Shadow
   (If OS-Kit worked on my box... Buanzo)
   File Version: ConioMMK-0.0.3-R2-MMK-0.0.7
   (c) 1999 Arturo Alberto Busleiman <buanzox@usa.net>
   Kernel Hacker/Programmer

   Portions (c) Luigi Sgro (ktk) and Bart Sekura (tinos)

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
   ============================================================================= */

#ifndef DEBUG
#include "/millennium/source/include/ConioMMK.h"
#else
#include "ConioMMK.h"
#endif

void
init_console_driver ()
{
  unsigned int i = 0;
  int *adapter;

/* with this routine we determine de video adapter type (color/mono) */

  adapter = (int *) 0x463;
  if (*adapter == CRT_INDEX)
    video[0] = (unsigned char *) MEM_VIDEO_COLOR;
  else
    video[0] = (unsigned char *) MEM_VIDEO_MONO;

  for (i = 0; i < NUMCONSOLE; i++)
    {
      consola[i].x = 0;
      consola[i].y = 0;
      consola[i].attr = 0x07;
      memset ((char *) &consola[i].videoimage, 0, sizeof (consola[i].videoimage));
    }

}


void
clrscr ()
{
  int i = 0;
  consola[actual_C].y = 0;
  consola[actual_C].x = 0;
  if (current_console == actual_C)
    {
      memset (&video, 0, sizeof (video));
      actualizar_cursor ();
    }
  else
    memset ((char *) &consola[actual_C].videoimage, 0, sizeof (consola[actual_C].videoimage));
}

void
screenscroll ()
{
  unsigned int i = 0;

  for (i = 0; i < 3840; i++)
    {
      if (actual_C == current_console)
	*video[i] = *video[i + 80 * 2];
      else
	consola[actual_C].videoimage[i] = consola[actual_C].videoimage[i + 80 * 2];
    }
  for (i = 3840; i < 4001; i++)
    if (actual_C == current_console)
      *video[i] = 0;
    else
      consola[actual_C].videoimage[i] = 0;
}

void
gotoxy (int x, int y)
{
  if ((x > 79) || (y > 24) || (x < 0) || (y < 0));
  else
    {
      consola[actual_C].x = x;
      consola[actual_C].y = y;
    }
  if (actual_C == current_console)
    actualizar_cursor ();
}

void
newline ()
{
  consola[actual_C].x = 0;
  if (consola[actual_C].y == 79)
    screenscroll ();
  else
    consola[actual_C].y++;
  if (current_console == actual_C)
    actualizar_cursor ();
}

void
actualizar_cursor ()
{
  static unsigned int curpos = 0;
  curpos = (consola[actual_C].y * 80) + consola[actual_C].x;
  outb (CRT_INDEX, CR_CURSOR_HI);
  outb (CRT_DATA, curpos >> 8);
  outb (CRT_INDEX, CR_CURSOR_LO);
  outb (CRT_DATA, curpos);
}

void
cputc (char caracter)
{
  unsigned int pos = 0;
  pos = (consola[actual_C].y * 160) + (consola[actual_C].x * 2);

  if (actual_C == current_console)
    {
      *video[pos] = caracter;
      *video[pos + 1] = consola[actual_C].attr;
    }
  else
    {
      consola[actual_C].videoimage[pos] = caracter;
      consola[actual_C].videoimage[pos + 1] = consola[actual_C].attr;
    }
  if (consola[actual_C].x == 79)
    newline ();
  else
    consola[actual_C].x++;

  if (actual_C == current_console)
    actualizar_cursor ();
}


void
setcolor (unsigned char bg, unsigned char fg)
{
  consola[actual_C].attr = (bg << 4) + fg;
}

int
hexEncode (unsigned long valor)
{
  int i, pos = 0;
  const char digitos[] = "0123456789ABCDEF";
  if (valor == 0)
    {
      buffer[0] = '0';
      buffer[1] = 'h';
      return 2;
    }

  i = 7;
  while ((valor >> 4 * i & 0xF) == 0)
    i--;

  do
    {
      buffer[pos++] = digitos[valor >> 4 * i-- & 0xF];
    }
  while (i >= 0);
  buffer[pos++] = 'h';
  return pos;
}

void
cprintf (char cadena[], unsigned long val)
{
  int i, len, tab, pos = 0;
  while (cadena[pos] != 0)
    {
      while (cadena[pos] != '\%' && cadena[pos] != '\n' && cadena[pos] != 0)
	cputc (cadena[pos++]);

      if (cadena[pos] == '\n')
	newline ();
      else if (cadena[pos] == '\%')
	switch (cadena[++pos])
	  {
	  case 'x':
	    {
	      len = hexEncode (val);
	      for (i = 0; i < len; i++)

		cputc (buffer[i]);
	      break;
	    }
	  default:
	    tab = cadena[pos] - '0';
	    if (cadena[++pos] != 'x')
	      return;
	    len = hexEncode (val);
	    for (i = 0; i < tab - len; i++)
	      cputc (' ');
	    for (i = 0; i < len; i++)
	      cputc (buffer[i]);
	  }
      else
	return;
      pos++;
    }
}



void
main_prueba ()
{
  setcolor (0, 7);
  gotoxy (10, 10);
  cprintf ("Al final parece que anda. La consola actual es la %x\n\n", actual_C);
  setcolor (0, 5);
  gotoxy (5, 22);
  cprintf ("La consola actual deberia ser la 0... espero que arriba diga lo mismo.", 0);
}

int
switch_console (unsigned char consolenum)
{
  unsigned int i = 0;
  if (consolenum > NUMCONSOLE - 1)
    return (E_NECON);		// no dynamic allocation yet :)

  for (i = 0; i < 4001; i++)
    consola[actual_C].videoimage[i] = *video[i];
  actual_C = consolenum;
  for (i = 0; i < 4001; i++)
    *video[i] = consola[actual_C].videoimage[i];
  return consolenum;
}
