/* ===========================================================================

   The Millennium Operating System
   This file is part of the Millennium Operating System source code.
   ConioMMK.h: Standard Function Calls to manipulate the console
   Prototypes, #defines, and memvars needed by ConioMMK.c

   File Version: ConioMMK-0.0.3-R1-MMK-0.0.7
   1999 Arturo Alberto Busleiman
   buanzox@usa.net
   Kernel Hacker/Programmer

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
   ============================================================================= */

/* This is the definition of a text console */

typedef struct
  {
    unsigned char x;
    unsigned char y;
    unsigned char attr;		// attributes: fg, bg, blink, etc

    char videoimage[4000];	// (25 * 80) * 2 = 4000
    // but we start from (0,0) not (1,1)
    // that's why we define 4000-2
    // 25 rows, 80 columns, 2 bytes(char+attr)
    // we use this to save the contents of the
    // console before switching to another one
    // and to restore this buffer to MemBegin addr
    // if actual_C != current_console (defined below) the routines
    // write directly to video memory, and save the contents of
    // it to the buffer when switch_console() is called
    // elseway they write to this buffer

  }
console;

// Error definitions for console (may be defined in errno.h, so we #ifndef)

#ifndef	E_NECON
#define	E_NECON	0x100
#endif

#ifndef	E_MSG_NECON
#define E_MSG_NECON	"Non existant text console"
#endif

// End Error definitions for text console

// Video port access #defines

#define CRT_INDEX       0x3D4
#define CRT_DATA        0x3D5
#define CR_CURSLINE1    0xA
#define CR_CURSLINE2    0xB
#define CR_CURSOR_HI    0xE
#define CR_CURSOR_LO    0xF

#define MEM_VIDEO_COLOR	0xB800
#define MEM_VIDEO_MONO	0xB000

unsigned char *video[4000];	// pointer to the screen
			// it is set-up in init_console_driver()

// miscellaneous #defines
#define NUMCONSOLE	7	// 7 text consoles sounds well, uh?

#define ATTR_BLINK	128	// 128 = 10000000 -> blinking

unsigned char actual_C = 0;	// this value of this variable has
				// NOTHING to do with the current
				// console. It does describe only
				// to which videobuffer the routines
				// must write. Obtained from the
				// processes information.

unsigned char current_console = 0;
// this is the console whose data is being mapped to video mem.
// a.k.a "current console" :)
// the kernel's process handler must set the value of this variable in 
// concordance with the console the process is running at.

console consola[NUMCONSOLE];

static unsigned char buffer[9];	// used by hexEncode()


// functions prototype's

void init_console_driver ();
void clrscr ();
void screenscroll ();
void gotoxy (int x, int y);
void newline ();
void actualizar_cursor ();
void cputc (char caracter);
void setcolor (unsigned char bg, unsigned char fg);
int hexEncode (unsigned long val);
void cprintf (char cadena[], unsigned long val);
int switch_console (unsigned char consolenum);
void main_prueba ();
